;----------------------------------------------------------
; e2_cal; Finite difference calc in 2-dim field   y
;         using parabolic wave equation           |
; Intro. IDL program. (2013): Chapt. 6.7          |
; Author:  Tadashi Ito                            ---->z
;----------------------------------------------------------            
;-----------------
pro e2_cal_init
;-----------------
common e2_cal_field, dyf,nyf,yfmax,dzf,nzf,zfmax,sint,szf,u,us
common e2_cal_const, im,eev,lamda,kv,r0
;--- some constants
im=dcomplex(0,1)
;---  transverse grid size (micron) 
yfmax = 20.
dyf = 0.04
nyf = long(yfmax/dyf)+1
;---  longitudinal grid size (micron)
zfmax = 10000. 
dzf = 0.04
nzf = long(zfmax/dzf)
;--- save frame interval
sint = 1000
szf = long(nzf/sint)+1
;--- field array of size nyf
u = make_array(nyf,/dcomplex)
us = make_array(szf+1,nyf,/dcomplex)
;--- x-ray (eV)
eev = 6000.
lamda = 1.239842/eev
kv = 2.*!dpi/lamda
r0 = -im/(2.*kv)*dyf/(dzf*dzf)/2.
;---slit width (4micron)
sw = 4. 
nsw = sw/dyf
;---aperture intensity 
ap = make_array(nsw,/dcomplex,value=1.d0)
yp=fix((nyf-1)/2-nsw/2)
u[yp] = ap
end
;----------------------
function u0, uu,ky,dy
;----------------------
;  get boundary value 
return, uu*exp(-dcomplex(0.,1.)*ky*dy)
end
;----------------
pro calc_field
;----------------
;--- calc 2-dim field using parabolic wave equation  
common e2_cal_field, dyf,nyf,yfmax,dzf,nzf,zfmax,sint,szf,u,us
common e2_cal_const, im,eev,lamda,kv,r0
;--- tridiagonal matrix and vector dd exclude boundary
upper = make_array(nyf-3,/dcomplex,value=-r0)
diag  = make_array(nyf-2,/dcomplex,value=1.+2*r0)
lower = make_array(nyf-3,/dcomplex,value=-r0)
dd     = make_array(nyf-2,/dcomplex)
;--- lu decomposition of the tridiagonal matrix
la_tridc, lower, diag, upper, u2, index
;--- calculation loop of finite-differences
sz=-1
for k = 0,nzf do begin
;--- store u in us
if (k mod sint) eq 0 then begin
    sz++ 
    us[sz,*]=u[*]
    print,sz,szf-1,format='(1x,I4,"/",I4)'     
    end
;--- set dd
for i=1,nyf-4 do begin
    dd[i]=r0*u[i]+(1.-2.*r0)*u[i+1]+r0*u[i+2]
    endfor
;--- include boundary value
ky=kv*(dyf*(nyf/2)/(dzf*k+1))
dd[0]=(1.-2.*r0)*u[1]+r0*u[2]+2.*r0*u0(u[1],ky,dyf)
dd[nyf-3]=r0*u[nyf-3]+(1.-2.*r0)*u[nyf-2]+2.*r0*u0(u[nyf-2],ky,dyf)
;--- solve equation m x res = dd
res = la_trisol(lower, diag, upper, u2, index, dd)
for i=1,nyf-2 do begin
    u[i]=res[i-1]
    endfor
;--- save boundary value
u[0] = u0(u[1],ky,dyf) 
u[nyf-1] = u0(u[nyf-2],ky,dyf)
endfor
end 
;---------------
;  button 1
;---------------
;  field calc
pro b1_ev,ev
common e2_cal_field, dyf,nyf,yfmax,dzf,nzf,zfmax,sint,szf,u,us
;---calculation  
calc_field
;---draw
d1_id=widget_info(ev.top, find_by_uname='d1')
widget_control, d1_id, get_value=w1_id  
wset, w1_id
device , decomposed = 0
loadct,7
tvscl,congrid(abs(us),!d.x_size,!d.y_size)
;---plot
d2_id=widget_info(ev.top, find_by_uname='d2')
widget_control, d2_id, get_value=w2_id  
wset, w2_id
plot,abs(us[szf-1,*])
end
;---------------
;  button 2
;---------------
;   re-draw
pro b2_ev,ev
common e2_cal_field, dyf,nyf,yfmax,dzf,nzf,zfmax,sint,szf,u,us
d1_id=widget_info(ev.top, find_by_uname='d1')
widget_control, d1_id, get_value=w1_id  
wset, w1_id
device , decomposed = 0
tvscl,congrid(abs(us),!d.x_size,!d.y_size)
end
;---------------
;  button 3
;---------------
;   change color
pro b3_ev,ev
xloadct
end
;---------------
;  button 4
;---------------
;   re-plot
pro b4_ev,ev
common e2_cal_field, dyf,nyf,yfmax,dzf,nzf,zfmax,sint,szf,u,us
s1_id=widget_info(ev.top, find_by_uname='s1')
widget_control, s1_id, get_value=cc
zp=c_to_i(cc)
d2_id=widget_info(ev.top, find_by_uname='d2')
widget_control, d2_id, get_value=w2_id  
wset, w2_id
plot,abs(us[zp,*])
end
;---------------
;  button write
;---------------
;   write tif file
pro b5_ev,ev
;---prepare output tiff filename
ofile='d:\e2_cal.tif'
d1_id=widget_info(ev.top, find_by_uname='d1')
widget_control, d1_id, get_value=w1_id
wset, w1_id
tvimg = tvrd(/true)
write_tiff, ofile, reverse(tvimg,3),xresol=350,yresol=350
end
;---------------
;  button quit
;---------------
pro bq_ev,ev
widget_control, /destroy, ev.top
end
;---------------
;  other events
;---------------
pro e2_cal_event,ev
end
;-------------------------------------------------
pro e2_cal
;    Main routine
;-------------------------------------------------
common e2_cal_field, dyf,nyf,yfmax,dzf,nzf,zfmax,sint,szf,u,us
common e2_cal_const, im,eev,lamda,kv,r0
;-----set initial parameters
e2_cal_init
;---base group
g0_id=widget_base(title='E2_cal', /row)
g1_id=widget_base(g0_id, /frame, /row)
g2_id=widget_base(g0_id, /frame, /column)
;---first group (draw) 
wsiz=400
d1_id=widget_draw(g1_id,uname='d1',xsize=wsiz,ysize=wsiz)
d2_id=widget_draw(g1_id,uname='d2',xsize=wsiz,ysize=wsiz)
;---second group (button)
b1_id =widget_button(g2_id,value='start',uname='b1',event_pro='b1_ev')
b2_id =widget_button(g2_id,value='re-draw',uname='b2',event_pro='b2_ev')
b3_id =widget_button(g2_id,value='xloadct',uname='b3',event_pro='b3_ev')
szf1=szf-1
s1_id =widget_slider(g2_id,value=szf1,minimum=0,maximum=szf1, $
title='Z_position',uname='s1')
b4_id =widget_button(g2_id,value='re-plot',uname='b4',event_pro='b4_ev')
b5_id =widget_button(g2_id,value='wr_tiff',uname='b5',event_pro='b5_ev')
bq_id =widget_button(g2_id,value='quit',uname='bq',event_pro='bq_ev')
;----realize
widget_control, g0_id, /realize
;-----call xmanager
xmanager, 'e2_cal', g0_id, /no_block
end